<?php
// Home Page Mode Component
//
// Copyright (c) 2011 Nagios Enterprises, LLC.  All rights reserved.
//  
// $Id: helpsystem.inc.php 182 2010-11-18 18:30:46Z egalstad $

require_once(dirname(__FILE__) . '/../componenthelper.inc.php');


// respect the name
$helpsystem_component_name = "helpsystem";

// run the initialization function
helpsystem_component_init();

////////////////////////////////////////////////////////////////////////
// COMPONENT INIT FUNCTIONS
////////////////////////////////////////////////////////////////////////

function helpsystem_component_init()
{
    global $helpsystem_component_name;

    $versionok = helpsystem_component_checkversion();

    $desc = "";
    if (!$versionok)
        $desc = "<br><b>" . gettext("Error: This component requires Nagios XI 2012R2.3 or later.") . "</b>";

    $args = array(
        COMPONENT_NAME => $helpsystem_component_name,
        COMPONENT_VERSION => '1.0.1',
        COMPONENT_DATE => '06/12/2014',
        COMPONENT_AUTHOR => "Nagios Enterprises, LLC",
        COMPONENT_DESCRIPTION => gettext("Enables per page help menus and videos throughout the Nagios XI interface."),
        COMPONENT_TITLE => "Help System",
        COMPONENT_PROTECTED => true,
        COMPONENT_TYPE => COMPONENT_TYPE_CORE,
        COMPONENT_CONFIGFUNCTION => "helpsystem_component_config_func"
    );

    register_component($helpsystem_component_name, $args);

    if ($versionok) {
        // configure callback to add help system js to head
        register_callback(CALLBACK_PAGE_HEAD, 'helpsystem_component_head_include');

        // add a menu link
        register_callback(CALLBACK_MENUS_INITIALIZED, 'helpsystem_addmenu');

        // add pagestart callback
        register_callback(CALLBACK_BODY_START, 'helpsystem_bodystart_callback');
    }

}


///////////////////////////////////////////////////////////////////////////////////////////
// VERSION CHECK FUNCTIONS
///////////////////////////////////////////////////////////////////////////////////////////

function helpsystem_component_checkversion()
{

    if (!function_exists('get_product_release'))
        return false;
    if (get_product_release() < 311)
        return false;

    return true;
}


///////////////////////////////////////////////////////////////////////////////////////////
// CONFIG FUNCTIONS
///////////////////////////////////////////////////////////////////////////////////////////

function helpsystem_component_config_func($mode = "", $inargs, &$outargs, &$result)
{

    // initialize return code and output
    $result = 0;
    $output = "";

    $component_name = "helpsystem";

    switch ($mode) {
        case COMPONENT_CONFIGMODE_GETSETTINGSHTML:

            // default settings
            $settings_default = array(
                "enabled" => 1,
                "allow_user_override" => 1,
            );

            // saved settings
            //set_option("helpsystem_component_options","");  // for debugging only
            $settings_raw = get_option("helpsystem_component_options");
            if ($settings_raw != "") {
                $settings_default = unserialize($settings_raw);
            }

            // settings passed to us
            $settings = grab_array_var($inargs, "settings", $settings_default);

            // fix checkboxes
            $settings["enabled"] = checkbox_binary(grab_array_var($settings, "enabled", ""));
            $settings["allow_user_override"] = checkbox_binary(grab_array_var($settings, "allow_user_override", ""));


            $output = '
			
	<div class="sectionTitle">' . gettext('Help System Settings') . '</div>
	
	<table>


	<tr>
	<td valign="top">
	<label for="enabled">' . gettext('Enable Help System:') . '</label><br class="nobr" />
	</td>
	<td>
	<input type="checkbox" class="checkbox" id="enabled" name="settings[enabled]" ' . is_checked($settings["enabled"], 1) . '>
<br class="nobr" />
	' . gettext('Enables the help system.') . '<br><br>
	</td>
	</tr>

	<tr>
	<td valign="top">
	<label for="enabled">' . gettext('Allow User-Override') . ':</label><br class="nobr" />
	</td>
	<td>
	<input type="checkbox" class="checkbox" name="settings[allow_user_override]" ' . is_checked($settings["allow_user_override"], 1) . '>
<br class="nobr" />
	' . gettext('Allow users to override help settings.') . '<br><br>
	</td>
	</tr>
	
	</table>

			';
            break;

        case COMPONENT_CONFIGMODE_SAVESETTINGS:

            // get variables
            $settings = grab_array_var($inargs, "settings", array());

            // fix checkboxes
            $settings["enabled"] = checkbox_binary(grab_array_var($settings, "enabled", ""));
            $settings["allow_user_override"] = checkbox_binary(grab_array_var($settings, "allow_user_override", ""));

            // validate variables
            $errors = 0;
            $errmsg = array();


            // handle errors
            if ($errors > 0) {
                $outargs[COMPONENT_ERROR_MESSAGES] = $errmsg;
                $result = 1;
                return '';
            }

            // save settings
            set_option("helpsystem_component_options", serialize($settings));
            set_option("helpsystem_component_options_configured", 1);

            break;

        default:
            break;

    }

    return $output;
}


///////////////////////////////////////////////////////////////////////////////////////////
// CALLBACK FUNCTIONS
///////////////////////////////////////////////////////////////////////////////////////////

function helpsystem_bodystart_callback($cbtype, &$cbargs)
{
    global $cfg;


    $debug = false;
    if ($debug)
        echo "BODYSTART<BR>";

    $hideoptions = grab_request_var('hideoptions', false);

    if ($hideoptions) { // help icon will be suppressed for scheduled reports
        if ($debug)
            echo "HIDEOPTIONS<BR>";
        return;
    }

    if (isset($cfg['disable_helpsystem']) && $cfg['disable_helpsystem'] == 1) {
        if ($debug)
            echo "HELP SYSTEM DISABLED<BR>";
        return;
    }
    // get our settings
    $settings_raw = get_option("helpsystem_component_options");
    if (empty($settings_raw)) {
        $settings = array(
            "enabled" => 1,
            "allow_user_override" => 1,
        );
    } else
        $settings = unserialize($settings_raw);

    // initial values
    $enabled = grab_array_var($settings, "enabled", 1);
    //$enabled=1;

    $global_configured = get_option("helpsystem_component_options_configured");
    $user_configured = get_user_meta(0, "helpsystem_component_options_configured");

    // bail out if we're not enabled or the component hasn't been configured...
    if ($enabled != 1) {
        if ($debug)
            echo "NOT ENABLED<BR>";
        return;
    }

    // GLOBAL SETTINGS
    $allow_user_override = grab_array_var($settings, "allow_user_override", "");

    // USER SETTINGS
    if ($user_configured == 1 && $allow_user_override == 1) {
        $settings_raw = get_user_meta(0, "helpsystem_component_options");
        if ($settings_raw != "") {
            $settings = unserialize($settings_raw);
        }
    }

    $enabled = grab_array_var($settings, "enabled", 1);
    if ($enabled != 1) {
        if ($debug)
            echo "NOT ENABLED BY USER<BR>";
        return;
    }

    //admin functionality
    $admin = file_exists(dirname(__FILE__) . '/admin') ? true : false;

    if (!$admin)
        $help_file = dirname(__FILE__) . "/help_system.xml";
    else
        $help_file = "http://assets.nagios.com/downloads/nagiosxi/helpsystem/helpsystem_admin.php?fullxml=true";


    $rq = $_REQUEST;
    $exclude_vars = array('nsp', 'search', 'searchButton', 'sortby', 'returnUrl', 'sort', 'orderby', 'token', 'id', 'pagelimit', 'sortlist', 'name_filter');
    foreach ($rq as $name => $value) {
        if (in_array($name, $exclude_vars))
            unset($rq[$name]);
    }

//    if ($cbargs['child'] == 1 && $help_id = helpsystem_get_help_match($help_file)){
    if ($cbargs['child'] == 1) { // always show help icon

        $help_id = helpsystem_get_help_match($help_file);

        // http://findicons.com/icon/66616/help_and_support
        if ($debug)
            echo "HELP ICON";
        echo '<div class="helpsystem_icon">
	<a onclick="help_system(' . $help_id . ')" href="#"><img src="' . get_base_url() . 'includes/components/helpsystem/images/help_and_support.png" width="24"></a>';
        if ($admin) {
            echo '<br/><a onclick="help_system_admin(' . $help_id . ');return false" href="#">Edit</a><script type="text/javascript">var helpsystem_request_vars=' . json_encode($rq) . '</script>';
        }
        echo '</div>';

    } else if ($cbargs['child'] == 1 && $admin) {
        if ($debug)
            echo "ADMIN ICON";
        echo '<div style="float: right;padding-left:5px">
	<a onclick="help_system_admin();return false" href="#">Create Help</a>
	</div><script type="text/javascript">var helpsystem_request_vars=' . json_encode($rq) . '</script>';

    }
}

function helpsystem_component_head_include($cbtype = '', $args = null)
{
    global $components;
    global $cfg;

    $debug = false;

    if (isset($cfg['disable_helpsystem']) && $cfg['disable_helpsystem'] == 1) {
        if ($debug)
            echo "DISABLED HEAD<BR>";
        return;
    }
    if ($debug)
        echo "HEAD<BR>";
    echo "<script type='text/javascript' src='" . get_base_url() . "includes/components/helpsystem/helpsysteminclude.js?" . $components['helpsystem'][COMPONENT_ARGS][COMPONENT_VERSION] . "'></script>";
    echo "<link rel='stylesheet' type='text/css' href='" . get_base_url() . "includes/components/helpsystem/helpsystem.css?" . $components['helpsystem'][COMPONENT_ARGS][COMPONENT_VERSION] . "' />";

}

function helpsystem_options_callback($cbtype, &$cbargs)
{

    // get our settings
    $settings_raw = get_option("helpsystem_component_options");
    if (empty($settings_raw)) {
        $settings = array(
            "enabled" => 1,
            "allow_user_override" => 1,
        );
    } else
        $settings = unserialize($settings_raw);

    // initial values
    $enabled = grab_array_var($settings, "enabled", 1);
    //$enabled=1;

    $global_configured = get_option("helpsystem_component_options_configured");
    $user_configured = get_user_meta(0, "helpsystem_component_options_configured");

    // bail out if we're not enabled or the component hasn't been configured...
    if ($enabled != 1) {
        return;
    }


    // GLOBAL SETTINGS
    $allow_user_override = grab_array_var($settings, "allow_user_override", 1);

    // USER SETTINGS
    if ($user_configured == 1 && $allow_user_override == 1) {
        $settings_raw = get_user_meta(0, "helpsystem_component_options");
        if ($settings_raw != "") {
            $user_settings = unserialize($settings_raw);
        } else
            $user_settings = array();
    }

}

function helpsystem_get_help_match($help_file)
{

    $xml = simplexml_load_file($help_file);
    //print_r($xml);
    $matches = array();
    foreach ($xml->item as $item) {
        //print_r($item);
        if (strpos($_SERVER['REQUEST_URI'], strval($item->request_uri)) !== FALSE) {
            $id = intval($item->id);
            // give 1 point for matching REQUEST_URI
            $matches[$id] = 0;
            if (empty($_REQUEST) && !isset($item->request)) {
                $matches[$id]++;
                continue;
            }
            if (!empty($_REQUEST) && isset($item->request)) {

                foreach ($item->request as $request) {

                    //print_r($request);

                    if ((!empty($request->name) && isset($_REQUEST[strval($request->name)])) && $_REQUEST[strval($request->name)] == strval($request->value)) {

                        // assign points
                        if (!empty($request->points))
                            // if points are specified in the XML assign that many points
                            $matches[$id] += intval($request->points);
                        else
                            // give 2 point for each match to the request name/value match
                            $matches[$id] += 2;
                    } else {
                        unset($matches[$id]);
                        break;
                    }

                } // end request loop
            }
        } // end REQUEST_URI if

    } // end item loop

//grab the key for the item with the most points 
    if (empty($matches) || max($matches) <= 0)
        return false;

    $maxs = array_keys($matches, max($matches));
    return $maxs[0];

}

function helpsystem_can_show_menu()
{
    global $cfg;

    if (isset($cfg['disable_helpsystem']) && $cfg['disable_helpsystem'] == 1) {
        return false;
    }

    // bail if we're not enabled
    $settings = @unserialize(get_option("helpsystem_component_options"));
    $enabled = grab_array_var($settings, "enabled", 1);
    $override = grab_array_var($settings, "allow_user_override", 1);

    if ($enabled == 0 || $override == 0) {
        //print_r($settings);
        return false;
    }

    return true;
}

function helpsystem_addmenu()
{

    $desturl = get_component_url_base("helpsystem");

    $mi = find_menu_item(MENU_ACCOUNT, "menu-account-accountinfo", "id");
    if ($mi == null)
        return;

    $order = grab_array_var($mi, "order", "");
    if ($order == "")
        return;

    $neworder = $order + .02;

    add_menu_item(MENU_ACCOUNT, array(
        "type" => "link",
        "title" => gettext("Help System"),
        "id" => "menu-account-helpsystemopts",
        "order" => $neworder,
        "opts" => array(
            "href" => $desturl . '/useropts.php',
        ),
        "function" => "helpsystem_can_show_menu",
    ));

}
