<?php
//
// Copyright (c) 2008-2014 Nagios Enterprises, LLC.  All rights reserved.
//
// $Id: components.php 3042 2014-12-15 16:05:10Z jomann $

//define("SKIPCOMPONENTS",1);  // skips auto-inclusion of components

require_once(dirname(__FILE__) . '/../includes/common.inc.php');

// Initialization stuff
pre_init();
init_session();
grab_request_vars();
decode_request_vars();

// Check prereqs and authentication
check_prereqs();
check_authentication(false);

// Only admins can access this page
if (is_admin() == false) {
    echo $lstr['NotAuthorizedErrorText'];
    exit();
}

// Route request
route_request();

function route_request()
{
    global $request;
    global $lstr;

    if (isset($request["download"]))
        do_download();
    else if (isset($request["checkupdates"])) {
        do_checkupdates();
    } else if (isset($request["upload"]))
        do_upload();
    else if (isset($request["delete"]))
        do_delete();
    else if (isset($request["config"])) {
        if (isset($request["cancelButton"]))
            show_components();
        else if (isset($request["update"]))
            do_configure();
        else
            show_configure();
    } else if (isset($request["installedok"]))
        show_components(false, $lstr['ComponentInstalledText']);
    else
        show_components();

    exit;
}

/**
 * @param bool   $error
 * @param string $msg
 */
function show_components($error = false, $msg = "")
{
    global $lstr;
    global $components;
    global $components_api_versions;

    $tmp = get_tmp_dir() . "/";
    $xmlcache = $tmp . 'components_api_versions.xml';
    if (file_exists($xmlcache)) {
        $components_api_versions = simplexml_load_file($xmlcache);
    }

    do_page_start(array("page_title" => $lstr['ManageComponentsPageTitle']), true);
?>

    <h1><?php echo $lstr['ManageComponentsPageHeader']; ?></h1>
    <?php display_message($error, false, $msg); ?>
    <div>
        <?php echo $lstr['ManageComponentsPageNotes']; ?>
        <p>
            <?php echo gettext("You can find additional components for Nagios XI at"); ?>
            <a href='http://exchange.nagios.org/directory/Addons/Components' target='_blank'>Nagios Exchange</a>.<br/>
        </p>
    </div>
    <br clear="all">
    <form enctype="multipart/form-data" action="components.php" method="post" style="margin-bottom:5px;">
        <div id='rightContainer' style="width: 150px; float: right;">
            <div class="bluebutton" style="margin-bottom:3px;">
                <a href="http://exchange.nagios.org/directory/Addons/Components" target="_blank"><?php echo gettext("Get Components"); ?></a>
            </div>
            <div class="bluebutton">
                <a href="?checkupdates=true"><?php echo gettext("Check for Updates"); ?></a>
            </div>
        </div>
        <input type="hidden" name="upload" value="1">
        <?php echo get_nagios_session_protector(); ?>
        <input type="hidden" name="MAX_FILE_SIZE" value="1000000"/>
        <label><?php echo $lstr["UploadNewComponentBoxText"]; ?>:</label><br>
        <input name="uploadedfile" class="textfield" type="file">
        <input type="submit" class="submitbutton" value="<?php echo $lstr['UploadComponentButton']; ?>"/>
    </form>
    <br clear="all">
    <table class="standardtable" style="width: 100%;">
        <thead>
        <tr>
            <th><?php echo gettext("Component"); ?></th>
            <th class="center" style="width: 54px"><?php echo gettext("Type"); ?></th>
            <th class="center" style="width: 60px;"><?php echo gettext("Settings"); ?></th>
            <th class="center" style="width: 70px;"><?php echo gettext("Actions"); ?></th>
            <th class="center" style="width: 60px;"><?php echo gettext("Version"); ?></th>
            <th class="center" style="width: 100px;"><?php echo $lstr['StatusTableHeader']; ?></th>
        </tr>
        </thead>
        <tbody>
        <?php
        $x = 0;

        $sorted_components = array();
        foreach ($components as $c) {
            $sorted_components[$c[COMPONENT_ARGS][COMPONENT_TITLE]] = $c;
        }
        ksort($sorted_components, SORT_STRING);

        foreach ($sorted_components as $carray) {
            if (!empty($carray[COMPONENT_ARGS][COMPONENT_TYPE])) {
                if ($carray[COMPONENT_ARGS][COMPONENT_TYPE] == COMPONENT_TYPE_CORE) {
                    continue;
                }
            }
            // Component may have just been deleted
            if (!file_exists(dirname(__FILE__) . "/../includes/components/" . $carray[COMPONENT_DIRECTORY])) {
                continue;
            }
            show_component($carray[COMPONENT_DIRECTORY], $carray[COMPONENT_ARGS][COMPONENT_NAME], $carray[COMPONENT_ARGS], $x);
            $x++;
        }
        ?>
        </tbody>
    </table>

    <h2><?php echo gettext("Core Components"); ?></h2>
    <table class="standardtable" style="width: 100%;">
        <thead>
        <tr>
            <th><?php echo gettext("Component"); ?></th>
            <th class="center" style="width: 54px"><?php echo gettext("Type"); ?></th>
            <th class="center" style="width: 60px;"><?php echo gettext("Settings"); ?></th>
            <th class="center" style="width: 60px;"><?php echo gettext("Version"); ?></th>
        </tr>
        </thead>
        <tbody>
        <?php
        $x = 0;
        foreach ($sorted_components as $carray) {
            if (empty($carray[COMPONENT_ARGS][COMPONENT_TYPE])) {
                continue;
            }
            // Component may have just been deleted
            if (!file_exists(dirname(__FILE__) . "/../includes/components/" . $carray[COMPONENT_DIRECTORY])) {
                continue;
            }
            show_component($carray[COMPONENT_DIRECTORY], $carray[COMPONENT_ARGS][COMPONENT_NAME], $carray[COMPONENT_ARGS], $x);
            $x++;
        }
        ?>
        </tbody>
    </table>

<?php
    do_page_end(true);
    exit();
}

/**
 * @param $component_dir
 * @param $component_name
 * @param $carray
 * @param $x
 */
function show_component($component_dir, $component_name, $carray, $x)
{
    global $lstr;
    global $components_api_versions;

    $rowclass = "";

    if (($x % 2) != 0)
        $rowclass .= " odd";
    else
        $rowclass .= " even";

    // grab variables
    $type = grab_array_var($carray, COMPONENT_TYPE, "");
    $title = grab_array_var($carray, COMPONENT_TITLE, "");
    $desc = grab_array_var($carray, COMPONENT_DESCRIPTION, "");
    $version = grab_array_var($carray, COMPONENT_VERSION, "");
    $date = grab_array_var($carray, COMPONENT_DATE, "");
    $author = grab_array_var($carray, COMPONENT_AUTHOR, "");
    $homepage = grab_array_var($carray, COMPONENT_HOMEPAGE, "");

    $configfunc = grab_array_var($carray, COMPONENT_CONFIGFUNCTION, "");
    $protected = grab_array_var($carray, COMPONENT_PROTECTED, false);

    echo "<tr class=" . $rowclass . ">";

    $displaytitle = $component_name;
    if ($title != "")
        $displaytitle = $title;

    echo "<td>";
    echo "<b>" . $displaytitle . "</b><br>";

    if ($desc != "")
        echo $desc . "<br>";

    if ($version != "")
        echo "Version: $version ";
    if ($date != "")
        echo "Date: $date ";
    if ($author != "")
        echo "Author: $author ";
    if ($homepage != "")
        echo "Website: <a href='$homepage' target='_blank'>$homepage<a/>";

    echo "</td>";

    echo "<td class='center'>";
    switch ($type) {
        case "core":
            echo "Core";
            break;
        default:
            echo "User";
            break;
    }
    echo "</td>";

    echo "<td class='center'>";
    if ($configfunc != "") {
        if (is_array($configfunc)) {
            echo "<a href='" . get_component_url_base($component_name) . "/". $configfunc['location'] ."'><img src='" . theme_image("editsettings.png") . "' alt=" . $lstr['EditSettingsAlt'] . "' title='" . $lstr['EditSettingsAlt'] . "'></a>";
        } else {
            echo "<a href='?config=" . $component_dir . "'><img src='" . theme_image("editsettings.png") . "' alt=" . $lstr['EditSettingsAlt'] . "' title='" . $lstr['EditSettingsAlt'] . "'></a>";
        }
    } else
        echo "-";
    echo "</td>";

    if ($type != 'core') {
        echo "<td class='center'>";
        if ($protected == false) {
            echo "<a href='?download=" . $component_dir . "'><img src='" . theme_image("download.png") . "' alt='" . $lstr['DownloadAlt'] . "' title='" . $lstr['DownloadAlt'] . "'></a> ";
            echo "<a href='?delete=" . $component_dir . "&nsp=" . get_nagios_session_protector_id() . "'><img src='" . theme_image("delete.png") . "' alt='" . $lstr['DeleteAlt'] . "' title='" . $lstr['DeleteAlt'] . "'></a>";
        } else
            echo "-";
        echo "</td>";
    }

    echo "<td class='center'>";
    if ($version != "")
        echo "$version";
    else
        echo "-";
    echo "</td>";

    if ($type != 'core') {
        if ($version != "" && isset($components_api_versions->$component_dir->version)) {

            if (version_compare($version, $components_api_versions->$component_dir->version, '<')) {
                echo "<td style='background-color:#B2FF5F' class='center'>";
                echo $components_api_versions->$component_dir->version . " Available<br/>";
                if ($components_api_versions->$component_dir->download != "")
                    echo "<a href='" . $components_api_versions->$component_dir->download . "'>Download</a>";
            } else {
                echo "<td class='center'>";
                echo "Up to date";
            }
        } else
            echo "<td class='center'>";
        echo "</td>";
    }

    echo "</tr>\n";
}


function do_download()
{
    global $lstr;

    // demo mode
    if (in_demo_mode() == true)
        show_components(true, $lstr['DemoModeChangeError']);

    $component_dir = grab_request_var("download");
    if (have_value($component_dir) == false)
        show_components();

    // clean the name
    $component_dir = str_replace("..", "", $component_dir);
    $component_dir = str_replace("/", "", $component_dir);
    $component_dir = str_replace("\\", "", $component_dir);

    $id = submit_command(COMMAND_PACKAGE_COMPONENT, $component_dir);
    if ($id <= 0)
        show_dashlets(true, $lstr['ErrorSubmittingCommandText']);
    else {
        for ($x = 0; $x < 40; $x++) {
            $status_code = -1;
            $result_code = -1;
            $args = array(
                "cmd" => "getcommands",
                "command_id" => $id,
            );
            $xml = get_backend_xml_data($args);
            if ($xml) {
                if ($xml->command[0]) {
                    $status_code = intval($xml->command[0]->status_code);
                    $result_code = intval($xml->command[0]->result_code);
                }
            }
            if ($status_code == 2) {
                if ($result_code == 0) {

                    // component was packaged, send it to user
                    $dir = get_tmp_dir();
                    $thefile = $dir . "/component-" . $component_dir . ".zip";

                    //chdir($dir);

                    $mime_type = "";
                    header('Content-type: ' . $mime_type);
                    header("Content-length: " . filesize($thefile));
                    header('Content-Disposition: attachment; filename="' . basename($thefile) . '"');
                    readfile($thefile);
                } else
                    show_components(true, $lstr['ComponentPackagingTimedOutText']);
                exit();
            }
            usleep(500000);
        }
    }

    exit();
}


function do_upload()
{
    global $lstr;

    //print_r($request);
    //exit();

    // demo mode
    if (in_demo_mode() == true)
        show_components(true, $lstr['DemoModeChangeError']);

    // check session
    check_nagios_session_protector();

    $target_path = get_tmp_dir() . "/";
    $component_file = basename($_FILES['uploadedfile']['name']);
    $target_path .= "component-" . $component_file;

    // log it
    send_to_audit_log(gettext("User installed component '") . $component_file . "'", AUDITLOGTYPE_CHANGE);

    //echo "TEMP NAME: ".$_FILES['uploadedfile']['tmp_name']."<BR>\n";
    //echo "TARGET: ".$target_path."<BR>\n";

    if (move_uploaded_file($_FILES['uploadedfile']['tmp_name'], $target_path)) {

        // fix perms
        chmod($target_path, 0770);
        chgrp($target_path, "nagios");

        $id = submit_command(COMMAND_INSTALL_COMPONENT, $component_file);
        if ($id <= 0)
            show_components(true, $lstr['ErrorSubmittingCommandText']);
        else {
            for ($x = 0; $x < 20; $x++) {
                $status_code = -1;
                $result_code = -1;
                $args = array(
                    "cmd" => "getcommands",
                    "command_id" => $id,
                );
                $xml = get_backend_xml_data($args);
                if ($xml) {
                    if ($xml->command[0]) {
                        $status_code = intval($xml->command[0]->status_code);
                        $result_code = intval($xml->command[0]->result_code);
                        $result_text = strval($xml->command[0]->result);
                    }
                }
                if ($status_code == 2) {
                    if ($result_code == 0) {
                        // redirect to show install message (so the list will include the new component)
                        header("Location: ?installedok");
                        exit();
                    } else {
                        //print_r($xml);
                        //echo "<BR>RESULT TEXT=$result_text<BR>";
                        $emsg = "";
                        if ($result_text != "")
                            $emsg .= " " . $result_text . "";
                        show_components(true, $lstr['ComponentInstallFailedText'] . $emsg);
                        //show_components(true,"ERROR=".$emsg);
                    }
                    exit();
                }
                usleep(500000);
            }
        }
        show_components(false, $lstr['ComponentScheduledForInstallationText']);
    } else {
        // error
        show_components(true, $lstr['ComponentUploadFailedText']);
    }

    exit();
}

function do_delete()
{
    global $lstr;

    // demo mode
    if (in_demo_mode() == true)
        show_components(true, $lstr['DemoModeChangeError']);

    // check session
    check_nagios_session_protector();

    $dir = grab_request_var("delete", "");

    // clean the filename
    $dir = str_replace("..", "", $dir);
    $dir = str_replace("/", "", $dir);
    $dir = str_replace("\\", "", $dir);

    if ($dir == "")
        show_components();

    // log it
    send_to_audit_log("User deleted component '" . $dir . "'", AUDITLOGTYPE_DELETE);

    $id = submit_command(COMMAND_DELETE_COMPONENT, $dir);
    if ($id <= 0)
        show_components(true, $lstr['ErrorSubmittingCommandText']);
    else {
        for ($x = 0; $x < 14; $x++) {
            $status_code = -1;
            $args = array(
                "cmd" => "getcommands",
                "command_id" => $id,
            );
            $xml = get_backend_xml_data($args);
            if ($xml) {
                if ($xml->command[0]) {
                    $status_code = intval($xml->command[0]->status_code);
                }
            }
            if ($status_code == 2) {
                show_components(false, $lstr['ComponentDeletedText']);
                exit();
            }
            usleep(500000);
        }
    }
    show_components(false, $lstr['ComponentScheduledForDeletionText']);
    exit();
}


/**
 * @param bool   $error
 * @param string $msg
 */
function show_configure($error = false, $msg = "")
{
    global $request;
    global $lstr;
    global $components;

    $dir = grab_request_var("config", "");

    // clean the filename
    $dir = str_replace("..", "", $dir);
    $dir = str_replace("/", "", $dir);
    $dir = str_replace("\\", "", $dir);

    $component_name = $dir;

    if ($component_name == "")
        show_components();

    $component = $components[$component_name];


    $title = grab_array_var($component[COMPONENT_ARGS], COMPONENT_TITLE, "");

    do_page_start(array("page_title" => $lstr['ConfigureComponentPageTitle'] . " - " . $title), true);

    ?>


    <h1><?php echo $title; ?></h1>


    <?php
    display_message($error, false, $msg);
    ?>

    <form method="post" action="">
    <input type="hidden" name="config" value="<?php echo encode_form_val($component_name); ?>">
    <input type="hidden" name="update" value="1">
    <?php echo get_nagios_session_protector(); ?>

    <?php
    // get component output
    $configfunc = grab_array_var($component[COMPONENT_ARGS], COMPONENT_CONFIGFUNCTION, "");
    if ($configfunc != "") {
        $inargs = $request;
        $outargs = array();
        $result = 0;
        $output = $configfunc(COMPONENT_CONFIGMODE_GETSETTINGSHTML, $inargs, $outargs, $result);
        echo $output;
    } else
        echo "Component function does not exist.";

    ?>

    <div id="formButtons">
    <input type="submit" class="submitbutton" name="submitButton" value="<?php echo $lstr['ApplySettingsButton']; ?>"/>
    <input type="submit" class="submitbutton" name="cancelButton" value="<?php echo $lstr['CancelButton']; ?>"/>

    <form>

<?php
    }

function do_checkupdates()
{

    $tmp = get_tmp_dir() . "/";
    $xmlcache = $tmp . 'components_api_versions.xml';
    $url = "http://api.nagios.com/product_versions/nagiosxi/components_api_versions.xml";
    //use proxy component?
    $proxy = false;
    if (have_value(get_option('use_proxy')))
        $proxy = true;

    $options = array(
        'return_info' => true,
        'method' => 'get',
        'timeout' => 10
    );

    // fetch the url
    $result = load_url($url, $options, $proxy);
    $getfile = trim($result["body"]);


    $error = false;
    // make sure we succeeded and the file is an appropriate length
    if ($getfile && strlen($getfile) > 3000) {
        file_put_contents($xmlcache, $getfile);
        $msg = "Component Versions Updated";
    } else {
        $error = true;
        $msg = gettext("Could not download component version list from Nagios Server, check Internet Connnectivity");
    }
    show_components($error, $msg);
}


/**
 * @param bool   $error
 * @param string $msg
 */
function do_configure($error = false, $msg = "")
{
    global $request;
    global $lstr;
    global $components;

    // demo mode
    if (in_demo_mode() == true)
        show_configure(true, $lstr['DemoModeChangeError']);

    // check session
    check_nagios_session_protector();

    $dir = grab_request_var("config", "");

    // clean the filename
    $dir = str_replace("..", "", $dir);
    $dir = str_replace("/", "", $dir);
    $dir = str_replace("\\", "", $dir);

    $component_name = $dir;

    if ($component_name == "")
        show_components();

    $component = $components[$component_name];

    // log it
    send_to_audit_log(gettext("User configured component '") . $component . "'", AUDITLOGTYPE_CHANGE);


    // save component settings
    $configfunc = grab_array_var($component[COMPONENT_ARGS], COMPONENT_CONFIGFUNCTION, "");
    if ($configfunc != "") {

        // pass request vars to component
        $inargs = $request;

        // initialize return values
        $outargs = array("test" => "test2");
        $result = 0;

        // tell component to save settings
        $configfunc(COMPONENT_CONFIGMODE_SAVESETTINGS, $inargs, $outargs, $result);

        // handle errors thrown by component
        if ($result != 0)
            show_configure(true, $outargs[COMPONENT_ERROR_MESSAGES]);

        // handle success
        else {
            $msg = $lstr['ComponentSettingsUpdatedText'];
            if (array_key_exists(COMPONENT_INFO_MESSAGES, $outargs))
                $msg = $outargs[COMPONENT_INFO_MESSAGES];
            show_configure(false, $msg);
        }
    } else
        echo gettext("Component function does not exist.");

    exit();
}

?>