<?php
//
//  Nagios Core Config Manager
//  Copyright (c) 2010-2014 Nagios Enterprises, LLC
//
//  File: ccm_log.inc.php
//  Desc: Displays the CCM log table.
//

/**
 * Handles the displaying of the CCM log management pages. Only available to Nagios XI
 * admins if the automatic login is enabled. New XI integration also allows for logging
 * of Apply Configurations ran through the nagioscorecfg component.
 */
function ccm_log()
{
    global $myConfigClass;
    global $myDataClass;
    global $ccmDB;
    
    // Request vars for page
    $limit = ccm_grab_request_var('pagelimit', '');
    $cmd = ccm_grab_request_var('cmd', '');
    $id = ccm_grab_request_var('id', '');
    $search = ccm_grab_request_var('search', '');
    $page = ccm_grab_request_var('page', 1);
    $submitted = ccm_grab_request_var('submitted', false);
    $delete = ccm_grab_request_var('delete_single', false);
    $delete_multi = ccm_grab_request_var('delete_multi', false);
    
    // Query the log database
    $query = 'SELECT COUNT(*) from tbl_logbook';
    $resultCount = $ccmDB->count_results($query);
        
    $retClass = 'invisible';
    $returnMessage = '';
        
    if ($submitted) {
        $errors = 0;
        require_once(INCDIR.'delete_object.inc.php');

        // Delete or search for any requested items
        if (intval($delete) != 0) {
            $returnMessage .= $ccmDB->delete_entry('logbook', 'id', $delete);
            $retClass = (strpos($returnMessage, 'failed') ? 'error' : 'success');
        }

        if ($delete_multi == 'true') {
            $startcount = $resultCount;
            $checks = ccm_grab_request_var('checked', array());
            $selectedcount = count($checks);
            foreach ($checks as $c) {
                $ccmDB->delete_entry('logbook', 'id', $c);
            }

            $diff = $ccmDB->count_results($query); 
            
            // Verify correct number deleted 
            if ($diff == ($startcount - $selectedcount)) {
                $returnMessage = "$selectedcount ".gettext("items deleted successfully")."!<br />";
                $retClass = 'success';
            } else {
                $returnMessage = ($startcount - $diff).gettext(" of ").$selectedcount.gettext("selected items were deleted").".<br />";
                $retClass = 'error';
            }
        }
    }

    // Page limit... if no post was submitted, use session limit or update session
    // limit if post was submitted with a new page limit
    if ($limit != '') { $_SESSION['limit'] = $limit; }
    if ($limit == '') { $limit = (isset($_SESSION['limit']) ? $_SESSION['limit'] : $_SESSION['default_limit']); }
    $limit = (($limit == 'none') ? $resultCount : $limit);

    // Initializing variables
    $query = 'SELECT * FROM tbl_logbook';
    if ($search != '') {
        $query .= "WHERE (`user` LIKE '%".$search."%' OR `entry` LIKE '%".$search."%' ";
    }
    $query .=" ORDER BY `time` DESC";  
    
    // Get the main result set 
    $sqlData = $ccmDB->query($query);
    $query = 'SELECT count(*) FROM tbl_logbook';
    if ($search != '') {
        $query .= "WHERE `user` LIKE '%".$search."%' OR `entry` LIKE '%".$search."%'"; 
    }
    $resultCount = $ccmDB->count_results($query);
            
    // Pagination
    $pagenumbers = '';
     
    // Do pagination if necessary 
    if ($resultCount > $limit) {
        $start = (($page == 1) ? 0 : (($page-1) * $limit));
        $end = ((($start + $limit) > $resultCount) ? $resultCount : ($start + $limit));
        $pagenumbers .= do_pagenumbers($page, $start, $limit, $resultCount, 'log');
    } else {   
        $start = 0; 
        $end = $resultCount;    
    }
    $start_display = $start+1;
    
    /////////////////BEGIN HTML BUILD //////////////////////
    $html = "
    <div id='contentWrapper'> 
        <h1 id='objectHeader'>".gettext('CCM Log Management')."</h1>
        <div id='returnContent' class='{$retClass}'>{$returnMessage}
            <div id='closeReturn'>
                <a href='javascript:void(0)' id='closeReturnLink' title='Close'>".gettext("Close")."</a>
            </div>
        </div>
        <div id='ccmtablewrapper'> 
            <form id='frmDatalist' method='post' action='index.php?cmd=admin&type=log'>    
                <div id='tableTopper'>
                    <div id='resultCounter' class='ccm-label'>".gettext("Displaying")." {$start_display}-{$end} ".gettext("of")." {$resultCount} ".gettext("results")."</div>
                    <div id='searchBox'>
                        <label for='search'>".gettext("Search").":</label>
                        <input type='text' name='search' id='search' value='{$search}' />
                        <input class='ccmButton' type='button' onclick='actionPic(\"admin\",\"\",\"\")' id='submitSearch' value='".gettext('Search')."' />
                        <input class='ccmButton' type='button' id='clear' name='clear' value='".gettext("Clear")."' />
                    </div>  
                    
                    <div class='clear'></div>
                </div>
                <table class='standardtable ccmtable'>                                  
                    <thead>
                        <tr>
                            <th class='tbl-checkbox'><input type='checkbox' class='checkbox' onclick='javascript:checkAll()' title='".gettext("Toggle All Checkboxes")."'></th>
                            <th class='tbl-time'>".gettext("Time")."</th>
                            <th class='tbl-ip'>".gettext("IP Address")."</th>
                            <th class='name_left_align'>".gettext("User")."</th>
                            <th class='name_left_align'>".gettext("Entry")."</th>
                            <th class='tbl-id'>ID</th>
                            <th style='width: 60px;'>".gettext("Actions")."</th>
                        </tr>
                    </thead>";
    
    //////////////////////////////////Table Rows Loop////////////////////////// 
    if ($resultCount != 0) {
        for ($i=$start; $i < $end; $i++)
        {
            $d = $sqlData[$i];
            //for table row class 
            $rowCounter % 2 == 1 ? $class = 'odd' : $class = 'even';
            $rowCounter++;      
            
            //begin heredoc string
            $entry = utf8_decode($d['entry']);
            $row=<<<ROW
                <tr>
                    <td class='tbl-checkbox'><input type='checkbox' class='checkbox' name='checked[]' value='{$d['id']}'  id='chbId{$rowCounter}' /></td>
                    <td class='tbl-time'>{$d['time']}</td>
                    <td class='tbl-ip'>{$d['ipadress']}</td>
                    <td class='name_left_align'>{$d['user']}</td>
                    <td class='name_left_align'>{$entry}</td>
                    <td class='tbl-id'>{$d['id']}</td>
                    <td class='action-icons'>
                        <img src='images/cross.png' alt='Delete' title='Delete' onclick="delete_single_log('{$d['id']}')" />
                    </td>
                </tr>
ROW;
            $html .= $row;
        }
    }
    /////////////////////////End Table Rows Loop ////////////////
    //handle empty table sets
    if($start==0 && $end ==0) $html.="<tr><td colspan='7'>".gettext("No results returned from logbook table")."</td></tr>";     
    //close out table after loop 
    $html .= "</table>";
    
    $tableControls="
    <div id='tableControlsBottom'>
        <input type='hidden' name='submitted' value='1'>
        <input type='hidden' name='delete_single' id='delete_single'>
        <div id='withCheckedDiv'>
            <label for='delete_multi'>".gettext("With Checked").": </label>
            <select name='delete_multi' id='delete_multi'>
              <option value='false'>&nbsp;</option>
              <option value='true'>".gettext("Delete")."</option> 
            </select>
            <input type='submit' class='ccmButton' value='".gettext("Go")."' />
        </div>
        ".$pagenumbers."
        <div id='pageLimitDiv'>
            <label for='pagelimit'>".gettext("Limit Results")."</label>
            <select name='pagelimit' id='pagelimit' onchange=\"actionPic('admin','','')\">
                <option id='limit15' value='15'>15</option>
                <option id='limit30' value='30'>30</option>
                <option id='limit50' value='50'>50</option>
                <option id='limit100' value='100'>100</option>
                <option id='limit250' value='250'>250</option>
                <option id='limitnone' value='none'>".gettext('None')."</option>
            </select>
            <script type='text/javascript'>
                limit ='".$limit."'; 
                $('#limit'+limit).attr('selected','selected');
            </script>
        </div>
    </div>
    </form>";

    $html .= $tableControls;
    $html .= "  </div>
            </div>";

    echo $html;
}