<?php
//
//  Nagios Core Config Manager
//  Copyright (c) 2010-2014 Nagios Enterprises, LLC
//
//  File: hidden_overlay_function.inc.php
//  Desc: Creates the HTML for all of the hidden overlays.
//

/**
 * Builds a hidden overlay div and populates values based on parameters given 
 *
 * @param string   $type nagios object type (host, service, command, etc)
 * @param string   $optionValue the DB fieldname for that objects name (host_name, service_description, template_name)
 * @param bool $BA boolean switch, are there two-way relationships possible for this object (host->hostgroup, hostgroup->host) 
 * @param bool $tplOpts boolean switch for showing template options 
 * @param string $fieldArray optional specification for which select list to use 
 *
 * @return string returns populated html select lists for the $type object
 */
function build_hidden_overlay($type, $optionValue, $BA=false, $tplOpts=false, $fieldArray='', $exactType='')
{
    global $FIELDS; 
    global $unique;

    $Title = ucfirst($type); 
    $Titles = ucfirst($type).'s'; 
    if ($fieldArray == '') {
        $fieldArray = 'sel'.$Title.'Opts'; 
    }

    $html = "<!-- ------------------------------------ {$Titles} --------------------- -->

    <div class='overlay' id='{$type}Box'>

    <div class='left'>
        <div class='listDiv'>
            <label for='sel{$Titles}'>{$Titles}</label>
            <select name='sel{$Titles}[]' size='9' multiple='multiple' class='lists' id='sel{$Titles}'>
                    <!-- option value is tbl ID -->
    "; 
    
    if($type=='hostservice') { //special case for hostService array //TODO: improve this 
        foreach($FIELDS['selHostServiceOpts'] as $key => $opt) {
            $html .= "<option ";
            if(in_array($key, $FIELDS['pre_hostservices_AB'])) $html .= "selected='selected' "; 
            if(in_array($key, $FIELDS['pre_hostservices_BA'])) $html .= "disabled='disabled' class='hiddenDependency' ";
            $html .= " id='".$unique++."' title='{$opt}' value='".$key."'>".$opt."</option>";
        }
    }
    elseif ($BA==true) {  //if there are two-way database relationships for this object 
        foreach($FIELDS[$fieldArray] as $opt) {
            $html .= '<option ';
            if(in_array($opt['id'], $FIELDS['pre_'.$type.'s_AB'])) $html .= "selected='selected' ";
            if(in_array($opt['id'], $FIELDS['pre_'.$type.'s_BA'])) {
                $html .= "disabled='disabled' class='hiddenDependency' title='Object has a relationship established elsewhere' ";
            } else if (grab_array_var($opt,'active',1) == 0 && $opt[$optionValue] != "*") {
                // If the object is not active we should turn it to disabled
                $html .= "disabled='disabled' ";
            }

            $html .= " id='".$unique++."' title='{$opt[$optionValue]}' value='".$opt['id']."'>".$opt[$optionValue].'</option>';

        }
    } else {
        // Only one-way DB relationships (i.e. service dependency)
        $pre_array = isset($FIELDS['pre_'.$type.'s_AB']) ? $FIELDS['pre_'.$type.'s_AB'] : $FIELDS['pre_'.$type.'s'] ;
        
        if ($exactType == "serviceescalation") {
            $uniq_services = array();
        }

        foreach($FIELDS[$fieldArray] as $opt) {

            // If it needs to be a unique service let's only display a service name once
            if ($exactType == "serviceescalation") {
                if (!in_array($opt[$optionValue], $uniq_services)) {
                    $uniq_services[] = $opt[$optionValue];
                } else {
                    continue;
                }
            }

            $html.= '<option ';
            $disabled = "";
            if (grab_array_var($opt,'active',1) == 0) { $disabled = " disabled='disabled'"; }
            if(in_array($opt['id'], $pre_array)) $html .= "selected='selected' orderid='".array_search($opt['id'], $pre_array)."'";

            // Display hostnames just for the service/host dependencies (and don't display in serviceescalation form)
            $name = $opt[$optionValue];
            if (($type == "servicedependency" || $type == "service") && $exactType != "serviceescalation") {
                if ($name != "*") {
                    $name = $opt['host_name'] . " - " . $name;
                }
            }

            $html .= " id='".$unique++."' title='{$name}' value='".$opt['id']."'".$disabled.">".$name.'</option>';
        }
    }
    $html .="  </select>
                </div>
                <div class='overlay-left-bottom'>
                        <div class='ccm-label'>\"=>\" ".gettext("Denotes object relationship elsewhere")."</div>
                  <p><a href='javascript:void(0)' class='linkBox' onclick='transferMembers(\"sel{$Titles}\", \"tbl{$Titles}\", \"{$type}s\")' title='Add'>".gettext("Add Selected")."</a></p>
                    ";
     if($tplOpts==true) { //template options     
        
        $radType = $type.'s';
        //deal with inconsistent DB naming convention in NagiosQL. Make sure we have the correct form field name
        $radType = (isset($FIELDS['contact_groups_tploptions']) && $type=='contactgroup') ? 'contact_groups' : $radType;
        $radType = (isset($FIELDS['host_name_tploptions']) && $type=='host') ? 'host_name' : $radType;
        $radType = (isset($FIELDS['hostgroup_name_tploptions']) && $type=='hostgroup') ? 'hostgroup_name' : $radType;
        $radType = ($type=='hostcommand') ? 'host_notification_commands': $radType;
        $radType = ($type=='servicecommand') ? 'service_notification_commands': $radType;
        
        //echo "RAD: $radType<br />\n";
        
        $html .= "               
            <div class='tplOptions'>
                <p class='ccm-label'>$Title ".gettext('Option')."s</p>
                <label style='margin-right: 5px;'>
                    <input type='radio' name='rad{$Title}' id='rad{$Title}0' value='0' ".@check($radType.'_tploptions', '0', true).">+
                </label>
                <label style='margin-right: 5px;'>
                    <input type='radio' name='rad{$Title}' id='rad{$Title}1' value='1' ". @check($radType.'_tploptions', '1', true). ">".gettext('null')."
                </label>
                <label style='margin-right: 5px;'>
                    <input type='radio' name='rad{$Title}' id='rad{$Title}2' value='2' ". @check($radType.'_tploptions', '2', true). ">".gettext('standard')."
                </label>
          </div>";
    } else {
        $html.="<div class='tplOptions empty'></div>";
    }
    
    $html .="
            <div class='closeOverlay'>
                <a class='linkBox' href='javascript:killOverlay(\"{$type}Box\")' title='Close'>".gettext("Close")."</a>
            </div>
        </div>
    </div>
    <!-- end leftBox -->
    <div class='right'>
                        
                  <table class='outputTable' id='tbl{$Titles}'>
                   <tr>
                    <th><div class='thMember'>".gettext("Assigned")."</div></th>
                    <th class='remove-center'>
                        <a title='Remove All' href='javascript:void(0)' onclick=\"removeAll('tbl{$Titles}')\">".gettext("Remove All")."</a>
                    </th>
                   </tr>
                  <!-- insert selected items here -->
                  </table>
       </div>
                                  
           <!-- $type radio buttons -->  
                                        
    </div> <!-- end {$type}box --> ";  

    return $html;
}


/**
 * Creates html overlay for the command-line test output
 *
 * @return string Returns the overlay's HTML
 */
function build_command_output_box()
{  
    $html = "
    <!-- ------------------------------------ Test Check Commands --------------------- -->
    <div class='overlay' id='commandOutputBox'>
        <h4>".gettext('Testing check from command line...')."</h4>
        <div id='command_output'>
            <img src='images/throbber1.gif' height='32' width='32' alt=''>
        </div>
        <p><a class='linkBox width100' href=\"javascript:killOverlay('commandOutputBox')\" title='Close'>".gettext("Close")."</a></p>
    </div>"; 
    
    return $html; 
}


/**
 * Creates html overlay for the free-variable definition form 
 *
 * @return string Returns html for the free variables overlay
 */ 
function build_variable_box()
{
    $html = "
    <!-- ------------------------------------ Free Variables --------------------- -->
    <div class='overlay' id='variableBox'>
        <div class='left'>
            <label for='txtVariablename'>".gettext("Variable name")." </label><br />
            <input type='text' name='txtVariablename' id='txtVariablename' style='width:225px' /><br />
            <label for='txtVariablename'>".gettext("Variable value")."</label>  <br />
            <input type='text' name='txtVariablevalue' id='txtVariablevalue' style='width:225px' /><br />
            <p>
                <a class='linkBox' href=\"javascript:void(0)\" onclick='insertDefinition(false, false)'>".gettext("Insert")."</a>
            </p>
        </div>
        <div class='right'>        
            <table class='outputTable' id='tblVariables'>
                <tr>
                    <th>".gettext("Variable Name")."</th>
                    <th>".gettext("Variable Definition")."</th>
                    <th>".gettext("Delete")."</th>
                </tr>
            </table>
        </div>
        <div class='closeOverlay'>
            <a class='linkBox' href=\"javascript:killOverlay('variableBox')\" title='Close'>".gettext("Close")."</a>
        </div>
    </div>";

    return $html;
}