<?php
// EVENT LOG REPORTS
//
// Copyright (c) 2008-2014 Nagios Enterprises, LLC.  All rights reserved.
//  
// $Id: eventlog.php 2933 2014-10-30 18:56:23Z jomann $

require_once(dirname(__FILE__) . '/../includes/common.inc.php');

// initialization stuff
pre_init();

// start session
init_session();

// grab GET or POST variables 
grab_request_vars();

// check prereqs
check_prereqs();

// check authentication
check_authentication(false);


route_request();

function route_request()
{

    // only admins can see the event log
    $user_id = grab_request_var("user_id");
    if (is_admin() == false && !get_user_meta($user_id, "authorized_for_monitoring_system")) {
        echo gettext("You are not authorized to view the event log");
        exit();
    }

    $mode = grab_request_var("mode", "");
    switch ($mode) {
        case "csv":
            get_eventlog_csv();
            break;
        case "pdf":
            get_eventlog_pdf();
            break;
        default:
            display_eventlog();
            break;
    }
}


///////////////////////////////////////////////////////////////////
// BACKEND DATA FUNCTIONS
///////////////////////////////////////////////////////////////////

// this function gets event log data in XML format from the backend
/**
 * @param $args
 *
 * @return SimpleXMLElement
 */
function get_eventlog_data($args)
{

    $xml = get_xml_logentries($args);

    return $xml;
}

///////////////////////////////////////////////////////////////////
// HELPER FUNCTIONS
///////////////////////////////////////////////////////////////////

// given an event log type, return corresponding image and text to use...
/**
 * @param $entrytype
 * @param $entrytext
 * @param $img
 * @param $text
 */
function get_eventlog_type_info($entrytype, $entrytext, &$img, &$text)
{

    // initial/default values
    $img = "info.png";
    $text = "";
    //return;

    // what type of log entry is this?  we change the image used for each line based on what type it is...
    switch ($entrytype) {
        case NAGIOSCORE_LOGENTRY_RUNTIME_ERROR:
            $img = "critical.png";
            $text = gettext("Runtime Error");
            break;
        case NAGIOSCORE_LOGENTRY_RUNTIME_WARNING:
            $img = "warning.png";
            $text = gettext("Runtime Warning");
            break;
        case NAGIOSCORE_LOGENTRY_VERIFICATION_ERROR:
            $img = "critical.png";
            $text = gettext("Verification Error");
            break;
        case NAGIOSCORE_LOGENTRY_VERIFICATION_WARNING:
            $img = "warning.png";
            $text = gettext("Verification Warning");
            break;
        case NAGIOSCORE_LOGENTRY_CONFIG_ERROR:
            $img = "critical.png";
            $text = gettext("Configuration Error");
            break;
        case NAGIOSCORE_LOGENTRY_CONFIG_WARNING:
            $img = "warning.png";
            $text = gettext("Configuration Warning");
            break;
        case NAGIOSCORE_LOGENTRY_PROCESS_INFO:
            $text = gettext("Process Information");
            break;
        case NAGIOSCORE_LOGENTRY_EVENT_HANDLER:
            $img = "action.gif";
            $text = gettext("Event Handler");
            break;
        case NAGIOSCORE_LOGENTRY_NOTIFICATION:
            $img = "notify.gif";
            $text = gettext("Notification");
            break;
        case NAGIOSCORE_LOGENTRY_EXTERNAL_COMMAND:
            $img = "command.png";
            $text = gettext("External Command");
            break;
        case NAGIOSCORE_LOGENTRY_HOST_UP:
            $img = "recovery.png";
            $text = gettext("Host Recovery");
            break;
        case NAGIOSCORE_LOGENTRY_HOST_DOWN:
            $img = "critical.png";
            $text = gettext("Host Down");
            break;
        case NAGIOSCORE_LOGENTRY_HOST_UNREACHABLE:
            $img = "critical.png";
            $text = gettext("Host Unreachable");
            break;
        case NAGIOSCORE_LOGENTRY_SERVICE_OK:
            $img = "recovery.png";
            $text = gettext("Service Recovery");
            break;
        case NAGIOSCORE_LOGENTRY_SERVICE_UNKNOWN:
            $img = "unknown.png";
            $text = gettext("Service Unknown");
            break;
        case NAGIOSCORE_LOGENTRY_SERVICE_WARNING:
            $img = "warning.png";
            $text = gettext("Service Warning");
            break;
        case NAGIOSCORE_LOGENTRY_SERVICE_CRITICAL:
            $img = "critical.png";
            $text = gettext("Service Critical");
            break;
        case NAGIOSCORE_LOGENTRY_PASSIVE_CHECK:
            $img = "passiveonly.gif";
            $text = gettext("Passive Check");
            break;
        case NAGIOSCORE_LOGENTRY_INFO_MESSAGE:

            $img = "info.png";
            $text = gettext("Information");

            if (strstr($entrytext, " starting...")) {
                $img = "start";
                $text = gettext("Program Start");
            } else if (strstr($entrytext, " shutting down...")) {
                $img = "stop.gif";
                $text = gettext("Program Stop");
            } else if (strstr($entrytext, "Bailing out")) {
                $img = "stop.gif";
                $text = gettext("Program Halt");
            } else if (strstr($entrytext, " restarting...")) {
                $img = "restart.gif";
                $text = gettext("Program Restart");
            } else if (strstr($entrytext, "SERVICE EVENT HANDLER:")) {
                $img = "serviceevent.gif";
                $text = gettext("Service Event Handler");
            } else if (strstr($entrytext, "HOST EVENT HANDLER:")) {
                $img = "hostevent.gif";
                $text = gettext("Host Event Handler");
            } else if (strstr($entrytext, " FLAPPING ALERT:")) {
                $img = "flapping.gif";
                if (strstr($entrytext, ";STARTED;"))
                    $text = gettext("Flapping Start");
                else if (strstr($entrytext, ";DISABLED;"))
                    $text = gettext("Flapping Disabled");
                else
                    $text = gettext("Flapping Stop");
            } else if (strstr($entrytext, " DOWNTIME ALERT:")) {
                $img = "downtime.gif";
                if (strstr($entrytext, ";STARTED;"))
                    $text = gettext("Scheduled Downtime Start");
                else if (strstr($entrytext, ";CANCELLED;"))
                    $text = gettext("Scheduled Downtime Cancelled");
                else
                    $text = gettext("Scheduled Downtime Stop");
            }

            break;
        case NAGIOSCORE_LOGENTRY_HOST_NOTIFICATION:
            $img = "notify.gif";
            $text = gettext("Host Notification");
            break;
        case NAGIOSCORE_LOGENTRY_SERVICE_NOTIFICATION:
            $img = "notify.gif";
            $text = gettext("Service Notification");
            break;

        default:
            $img = "info.png";
            break;
    }
}


///////////////////////////////////////////////////////////////////
// REPORT GENERATION FUCNTIONS
///////////////////////////////////////////////////////////////////

// this function displays event log data in HTML
function display_eventlog()
{
    global $request;

    // makes sure user has appropriate license level
    licensed_feature_check();

    // get values passed in GET/POST request
    $page = grab_request_var("page", 0);
    $records = grab_request_var("records", 25);
    $reportperiod = grab_request_var("reportperiod", "last24hours");
    $startdate = grab_request_var("startdate", "");
    $enddate = grab_request_var("enddate", "");
    $search = grab_request_var("search", "");
    $hideoptions = grab_request_var("hideoptions", 0);
    $manual_run = grab_request_var("manual_run", 0);

    // Do not do any processing unless we have default report running enabled
    $disable_report_auto_run = get_option("disable_report_auto_run", 0);
    if ($manual_run == 1 || !empty($page) || $disable_report_auto_run == 0) {

        // fix custom dates
        if ($reportperiod == "custom") {
            if ($enddate == "") {
                $enddate = strftime("%c", time());
            }
            if ($startdate == "") {
                $startdate = strftime("%c", time() - (60 * 60 * 24));
                $enddate = strftime("%c", time());
            }
        }

        // start/end times must be in unix timestamp format
        // if they weren't specified, default to last 24 hours
        //$endtime=grab_request_var("endtime",time());
        //$starttime=grab_request_var("starttime",$endtime-(24*60*60));

        // determine start/end times based on period
        get_times_from_report_timeperiod($reportperiod, $starttime, $endtime, $startdate, $enddate);

        // get XML data from backend - the most basic example
        // this would return all records (no paging), so it could be used for CSV export
        /*
        $args=array(
            "starttime" => $starttime,
            "endtime" => $endtime,
            );
        $xml=get_eventlog_data($args);
        */
        // NOTES:
        // TOTAL RECORD COUNT (FOR PAGING): if you wanted to get the total count of records in a given timeframe (instead of the records themselves), use this:
        /**/
        $args = array(
            "starttime" => $starttime,
            "endtime" => $endtime,
            "totals" => 1,
        );
        if ($search)
            $args["logentry_data"] = "lk:" . $search;
        $xml = get_eventlog_data($args);
        $total_records = 0;
        if ($xml)
            $total_records = intval($xml->recordcount);

        // determine paging information
        $args = array(
            "reportperiod" => $reportperiod,
            "startdate" => $startdate,
            "enddate" => $enddate,
            "starttime" => $starttime,
            "endtime" => $endtime,
            "search" => $search,
        );
        $pager_results = get_table_pager_info("", $total_records, $page, $records, $args);
        $first_record = (($pager_results["current_page"] - 1) * $records);

        /**/
        // SPECIFIC RECORDS (FOR PAGING): if you want to get specific records, use this type of format:
        /**/
        $args = array(
            "starttime" => $starttime,
            "endtime" => $endtime,
            "records" => $records . ":" . $first_record,
        );
        if ($search)
            $args["logentry_data"] = "lk:" . $search;
        $xml = get_eventlog_data($args);
        //$xml=null;
        /**/

    } // End DISABLED LOADING

    // start the HTML page
    do_page_start(array("page_title" => gettext("Event Log")), true);

    if ($hideoptions) {
        // default logo stuff
        $logo = "nagiosxi-logo-small.png";
        $logo_alt = get_product_name();

        // use custom logo if it exists
        $logosettings_raw = get_option("custom_logo_options");
        if ($logosettings_raw == "") {
            $logosettings = array();
        } else {
            $logosettings = unserialize($logosettings_raw);
        }

        $custom_logo_enabled = grab_array_var($logosettings, "enabled");
        if ($custom_logo_enabled == 1) {
            $logo = grab_array_var($logosettings, "logo", $logo);
            $logo_alt = grab_array_var($logosettings, "logo_alt", $logo_alt);
        }
        ?>

        <div style="padding-bottom: 10px;">
            <div style="float: left; margin-right: 30px;">
                <img src="<?php echo get_base_url(); ?>images/<?php echo $logo; ?>" border="0"
                     alt="<?php echo $logo_alt; ?>" title="<?php echo $logo_alt; ?>">
            </div>
            <div style="float: left; height: 44px;">
                <div
                    style="font-weight: bold; font-size: 22px; padding-bottom: 4px;"><?php echo gettext("Event Log"); ?></div>
                <div><?php echo gettext("From"); ?>:
                    <strong><?php echo get_datetime_string($starttime, DT_SHORT_DATE_TIME, DF_AUTO, "null"); ?></strong> <?php echo gettext("To"); ?>
                    : <strong><?php echo get_datetime_string($endtime, DT_SHORT_DATE_TIME, DF_AUTO, "null"); ?></strong>
                </div>
            </div>
            <div style="clear:both;"></div>
        </div>

    <?php
    } else {
        ?>
        <h1><?php echo gettext("Event Log"); ?></h1>

        <form method="get" action="<?php echo htmlentities($_SERVER["REQUEST_URI"]); ?>">

        <div class="reportexportlinks">
            <?php echo get_add_myreport_html("Event Log", $_SERVER["REQUEST_URI"], array()); ?>
            <?php
            $url = "?1";
            foreach ($request as $var => $val)
                $url .= "&" . urlencode($var) . "=" . urlencode($val);
            ?>
            <a href="<?php echo $url; ?>&mode=csv" alt="<?php echo gettext("Export As CSV"); ?>"
               title="<?php echo gettext("Export As CSV"); ?>"><img src="<?php echo theme_image("csv.png"); ?>"></a>
            <a href="<?php echo $url; ?>&mode=pdf" alt="<?php echo gettext("Download As PDF"); ?>"
               title="<?php echo gettext("Download As PDF"); ?>"><img src="<?php echo theme_image("pdf.png"); ?>"></a>
        </div>

        <div class="reportsearchbox">
            <input type="text" size="15" name="search" id="searchBox" value="<?php echo encode_form_val($search); ?>"
                   placeholder="<?php echo gettext("Search..."); ?>" class="textfield"/>
        </div>

        <?php
        $auto_start_date = get_datetime_string(strtotime('yesterday'), DT_SHORT_DATE);
        $auto_end_date = get_datetime_string(strtotime('Today'), DT_SHORT_DATE);
        ?>

        <script type="text/javascript">
            $(document).ready(function () {
                $('#startdateBox').click(function () {
                    $('#reportperiodDropdown').val('custom');
                    if ($('#startdateBox').val() == '' && $('#enddateBox').val() == '') {
                        $('#startdateBox').val('<?php echo $auto_start_date;?>');
                        $('#enddateBox').val('<?php echo $auto_end_date;?>');
                    }
                });
                $('#enddateBox').click(function () {
                    $('#reportperiodDropdown').val('custom');
                    if ($('#startdateBox').val() == '' && $('#enddateBox').val() == '') {
                        $('#startdateBox').val('<?php echo $auto_start_date;?>');
                        $('#enddateBox').val('<?php echo $auto_end_date;?>');
                    }
                });

                showhidedates();

                $('#reportperiodDropdown').change(function () {
                    showhidedates();
                });

                function showhidedates() {
                    if ($('#reportperiodDropdown').val() == 'custom')
                        $('#customdates').show();
                    else
                        $('#customdates').hide();
                }

            });
        </script>

        <div class="reporttimepicker">
            <div style="float: left; padding-right: 10px;">
                <?php echo gettext("Period"); ?>&nbsp;
                <select id='reportperiodDropdown' name="reportperiod">
                    <?php
                    $tp = get_report_timeperiod_options();
                    foreach ($tp as $shortname => $longname) {
                        echo "<option value='" . $shortname . "' " . is_selected($shortname, $reportperiod) . ">" . $longname . "</option>";
                    }
                    ?>
                </select>
            </div>

            <div id="customdates" class="hide" style="float: left;">
                &nbsp;<?php echo gettext("From"); ?>&nbsp;
                <input class="textfield" type="text" id='startdateBox' name="startdate"
                       value="<?php echo $startdate; ?>" size="16"/>

                <div class="reportstartdatepicker"><img src="<?php echo theme_image("calendar_small.png"); ?>"></div>
                <div id="startdatepickercontainer"></div>
                &nbsp;<?php echo gettext("To"); ?>&nbsp;
                <input class="textfield" type="text" id='enddateBox' name="enddate" value="<?php echo $enddate; ?>"
                       size="16"/>

                <div class="reportenddatepicker"><img src="<?php echo theme_image("calendar_small.png"); ?>"></div>
                <div id="enddatepickercontainer"></div>
                &nbsp;
            </div>
            <input type='submit' class='reporttimesubmitbutton' name='reporttimesubmitbutton'
                   value='<?php echo gettext("Update"); ?>'>

            <!-- Set a variable to let us know it's okay to run this -->
            <input type="hidden" name="manual_run" value="1">

        </div>

        <?php
        // Die right here if we don't want to auto-load the page
        if ($disable_report_auto_run == 1 && ($manual_run == 0 && empty($page))) {
            die();
        }
        ?>

        <div style="clear: left; padding-top: 10px;">
            <?php echo gettext("Report Covers"); ?>&nbsp;<?php echo gettext("From"); ?>:
            <b><?php echo get_datetime_string($starttime, DT_SHORT_DATE_TIME, DF_AUTO, "null"); ?></b> <?php echo gettext("To"); ?>
            : <b><?php echo get_datetime_string($endtime, DT_SHORT_DATE_TIME, DF_AUTO, "null"); ?></b>
        </div>

    <?php } // end hideoptions ?>

    <div class="recordcounttext">
        <?php
        $clear_args = array(
            "reportperiod" => $reportperiod,
            "startdate" => $startdate,
            "enddate" => $enddate,
            "starttime" => $starttime,
            "endtime" => $endtime,
            //"search" => $search,
        );
        echo table_record_count_text($pager_results, $search, true, $clear_args);
        ?>
    </div>

    <?php if ($hideoptions) { ?>
    <div class="eventlogentries" style="border: none; padding: 0;">
    <style type="text/css">
        table.standardtable {
            font-size: 9px;
        }

        table.standardtable thead tr th {
            font-size: 10px;
        }
    </style>
    <table class="standardtable eventlogtable">
    <thead>
    <tr>
        <th><?php echo gettext("Type"); ?></th>
        <th><?php echo gettext("Date / Time"); ?></th>
        <th><?php echo gettext("Information"); ?></th>
    </tr>
    </thead>
    <tbody>
<?php
} else {
    echo '<div class="eventlogentries">';
} // end hideoptions

    if ($xml) {
        foreach ($xml->logentry as $le) {

            // what type of log entry is this?  we change the image used for each line based on what type it is...
            $entrytype = intval($le->logentry_type);
            $entrytext = strval($le->logentry_data);

            get_eventlog_type_info($entrytype, $entrytext, $type_img, $type_text);

            if ($hideoptions) {
                echo "<tr>";
                echo "<td>" . $type_text . "</td>";
                echo "<td nowrap>" . $le->entry_time . "</td>";
                echo "<td>" . str_replace('&apos;', "'", html_entity_decode($entrytext)) . "</td>";
                echo "</tr>";
            } else {
                echo "<div class='logentry logentry-" . $entrytype . "'><span class='logentrytype'><img src='" . nagioscore_image($type_img) . "' alt='" . $type_text . "' title='" . $type_text . "'></span><span class='logentrytime'>" . $le->entry_time . "</span><span class='logentrydata'>" . $entrytext . "</span></div>";
            }
        }
    }

    if ($hideoptions) {
        ?>

        </tbody>
        </table>
        </div>


    <?php } else { ?>
        </div>

        <div class='recordpagerlinks'>
            <form method="get" action="">
                <?php table_record_pager($pager_results); ?>
            </form>
        </div>

        </form>

    <?php
    } // end hideoptions


    // closes the HTML page
    do_page_end(true);
}


// this function gets the XML records of event log data for multiple
// output formats (CSV, PDF, HTML)
/**
 * @return SimpleXMLElement
 */
function get_eventlog_xml()
{
    global $lstr;

    // makes sure user has appropriate license level
    licensed_feature_check();

    // get values passed in GET/POST request
    $reportperiod = grab_request_var("reportperiod", "last24hours");
    $startdate = grab_request_var("startdate", "");
    $enddate = grab_request_var("enddate", "");
    $search = grab_request_var("search", "");

    // fix search
    if ($search == $lstr['SearchBoxText'])
        $search = "";


    // determine start/end times based on period
    get_times_from_report_timeperiod($reportperiod, $starttime, $endtime, $startdate, $enddate);

    // get XML data from backend - the most basic example
    // this will return all records (no paging), so it can be used for CSV export
    $args = array(
        "starttime" => $starttime,
        "endtime" => $endtime,
    );
    if ($search)
        $args["logentry_data"] = "lk:" . $search;
    $xml = get_eventlog_data($args);
    return $xml;
}

// this function generates a CSV file of event log data
function get_eventlog_csv()
{
    $xml = get_eventlog_xml();

    // output header for csv
    //header('Content-type: application/text');
    //header("Content-length: " . filesize($thefile));
    //header('Content-Disposition: attachment; filename="'.basename($thefile).'"');
    header("Content-type: application/octet-stream");
    header("Content-Disposition: attachment; filename=\"eventlog.csv\"");

    // column definitions
    //echo "time,type,data\n";
    echo "type,time,information\n";

    if ($xml) {
        foreach ($xml->logentry as $le) {

            // what type of log entry is this?  we change the image used for each line based on what type it is...
            $entrytype = intval($le->logentry_type);
            $entrytext = strval($le->logentry_data);

            // image
            get_eventlog_type_info($entrytype, $entrytext, $type_img, $type_text);

            //echo $le->entry_time.",".$entrytype.",".$entrytext."\n";
            echo $type_text . "," . $le->entry_time . ",\"" . str_replace('&apos;', "'", html_entity_decode($entrytext)) . "\"\n";
        }
    }
}


// This function generates a PDF of event log data.
// Requires the "fpdp.php" library and the "fonts" directory for font metrics.
function get_eventlog_pdf()
{
    global $cfg;

    //get user name
    $username = $_SESSION["username"];
    // get backend ticket
    $backend_ticket = get_user_attr(0, "backend_ticket");

    // Assemble actual URL that will be gotten
    $prefix = ($cfg['use_https'] == true) ? 'https' : 'http';
    $uri = str_replace("mode=pdf", "hideoptions=1", $_SERVER["REQUEST_URI"]);
    $fullurl = "{$prefix}://127.0.0.1{$uri}";
    //echo "FULLURL: $fullurl\n";

    $urlparts = parse_url($fullurl);

    $newurl = "";
    $newurl .= get_internal_url() . "/reports/eventlog.php";
    $newurl .= "?";
    if (isset($urlparts['query']))
        $newurl .= $urlparts['query'];

    $newurl .= "&username=" . $username;
    $newurl .= "&ticket=" . $backend_ticket;

    //echo "NEWURL: $newurl\n";

    // Add language to url
    $language = $_SESSION['language'];
    $newurl .= "&locale=" . $language;

    // Temporary fix for num records
    $newurl .= "&records=100000"; // Max: 100,000 alert producers

    // Do page rendering

    $aurl = $newurl;

    $afile = "page.pdf";
    $fname = get_tmp_dir() . "/scheduledreport-" . $username . "-" . time() . "-" . $afile;

    $cmdft = '--footer-spacing 3 --margin-bottom 15mm --footer-font-size 9 --footer-right "Page [page] of [toPage]" --footer-left "' . get_datetime_string(time(), DT_SHORT_DATE_TIME, DF_AUTO, "null") . '"';

    // --orientation landscape
    $cmd = "/usr/bin/wkhtmltopdf --disable-smart-shrinking --no-outline {$cmdft} '{$aurl}' '{$fname}' 2>&1";
    @exec($cmd);

    if (!file_exists($fname)) {
        echo "\n\n************\nERROR: Failed to render URL '" . $aurl . "' as '" . $fname . "'\n************\n\n";
    } else {
        // We'll be outputting a PDF
        header('Content-type: application/pdf');
        header('Content-Disposition: inline; filename="eventlog.pdf"');

        // The PDF source is in original.pdf
        readfile($fname);
        unlink($fname);
    }

    /*
    global $lstr;

    //require_once(dirname(__FILE__).'/../includes/fpdf/fpdf.php');
    require_once(dirname(__FILE__).'/../includes/fpdf/fpdf_alpha.php');
    require_once(dirname(__FILE__).'/../includes/fpdf/nagiospdf.php');

    // get values passed in GET/POST request
    $page=grab_request_var("page",1);
    $records=grab_request_var("records",25);
    $reportperiod=grab_request_var("reportperiod","last24hours");
    $startdate=grab_request_var("startdate","");
    $enddate=grab_request_var("enddate","");
    $search=grab_request_var("search","");

    // fix search
    if($search==$lstr['SearchBoxText'])
        $search="";

    // determine start/end times based on period
    get_times_from_report_timeperiod($reportperiod,$starttime,$endtime,$startdate,$enddate);


    $date_text="".get_datetime_string($starttime,DT_SHORT_DATE_TIME,DF_AUTO,"null")." To ".get_datetime_string($endtime,DT_SHORT_DATE_TIME,DF_AUTO,"null")."";

    // get event log entries in XML
    $xml = get_eventlog_xml();

    $records=0;
    if($xml){
        $records=intval($xml->recordcount);
        }

    $pdf=new NagiosReportPDF();
    $pdf->page_title='Event Log';
    $pdf->page_subtitle=$date_text."\n\n".gettext("Showing")." ".$records." ".gettext("Log Entries");
    if($search!="")
        $pdf->page_subtitle.=" ".gettext("Matching")." '".$search."'";

    $pdf->AliasNbPages();
    $pdf->AddPage();
    $pdf->SetFont("Helvetica", "", 7);


    $fs=' style="times" size="10" style="bold"';

    $html='
    <table border="1" align="center">
    <tr	bgcolor="#cccccc" repeat>
    <td '.$fs.'>'.gettext("Type").'</td>
    <td '.$fs.'>'.gettext("Date / Time").'</td>
    <td '.$fs.'>'.gettext("Information").'</td>
    </tr>
    ';

    $fs=' size="8" style=""';

    $type_image="";
    $type_text="";

    if($xml){
        foreach($xml->logentry as $le){

            // what type of log entry is this?  we change the image used for each line based on what type it is...
            $entrytype=intval($le->logentry_type);
            $entrytext=strval($le->logentry_data);

            // image
            get_eventlog_type_info($entrytype,$entrytext,$type_img,$type_text);
            //$pdf->Image(get_base_dir().'/includes/components/nagioscore/ui/images/'.$type_img,$pdf->GetX(),$pdf->GetY(),4,4);

            //$text = $le->entry_time . " " . $entrytext;
            //<img src="'.get_base_dir().'/includes/components/nagioscore/ui/images/'.$type_img.'">

            $html.='<tr><td '.$fs.'>'.$type_text.'</td><td '.$fs.'>'.$le->entry_time.'</td><td '.$fs.'>'.str_replace('&apos;', "'",html_entity_decode($entrytext)).'</td></tr>';

            // MultiCell handles line breaks, keeps text from overflowing.
            //$pdf->SetX($pdf->GetX()+4);
            //$pdf->MultiCell(0, 6, $text, 0, "L");
            }
        }

    $html.='</table>';

    //$pdf->setfont('times','I',8);
    $pdf->htmltable($html);

    $pdf->Output("eventlog.pdf", "I");
    */
}

?>
