<?php
// ORACLE QUERY CONFIG WIZARD
//
// Copyright (c) 2010 Nagios Enterprises, LLC.  All rights reserved.
//  
// $Id: mysqlquery.inc.php 688 2011-07-27 11:44:14Z egalstad $

include_once(dirname(__FILE__) . '/../configwizardhelper.inc.php');

// run the initialization function
oraclequery_configwizard_init();

function oraclequery_configwizard_init()
{

    $name = "oraclequery";

    $args = array(
        CONFIGWIZARD_NAME => $name,
        CONFIGWIZARD_VERSION => "1.3",
        CONFIGWIZARD_TYPE => CONFIGWIZARD_TYPE_MONITORING,
        CONFIGWIZARD_DESCRIPTION => gettext("Monitor a Oracle Query"),
        CONFIGWIZARD_DISPLAYTITLE => "Oracle Query",
        CONFIGWIZARD_FUNCTION => "oraclequery_configwizard_func",
        CONFIGWIZARD_PREVIEWIMAGE => "oraclequery.png",
    );

    register_configwizard($name, $args);
}


/**
 * @param string $mode
 * @param null   $inargs
 * @param        $outargs
 * @param        $result
 *
 * @return string
 */
function oraclequery_configwizard_func($mode = "", $inargs = null, &$outargs, &$result)
{

    $wizard_name = "oraclequery";

    // initialize return code and output
    $result = 0;
    $output = "";

    // initialize output args - pass back the same data we got
    $outargs[CONFIGWIZARD_PASSBACK_DATA] = $inargs;


    switch ($mode) {
        case CONFIGWIZARD_MODE_GETSTAGE1HTML:

            $address = grab_array_var($inargs, "address", "");
            $port = grab_array_var($inargs, "port", "1521");
            $sid = grab_array_var($inargs, "sid", "");
            $username = grab_array_var($inargs, "username", "");
            $password = grab_array_var($inargs, "password", "");
            $database = grab_array_var($inargs, "database", "system");

            //sanity check for wizard requirements
            $sanity = '';
            if (!file_exists('/usr/lib/oracle') && !file_exists('/usr/lib64/oracle')) {
                $sanity .= "<div style='color:red'><strong>" . gettext("WARNING") . ":</strong> " . gettext("Oracle libraries do not appear to be installed. See the") . "
				<a href='http://assets.nagios.com/downloads/nagiosxi/docs/Oracle_Plugin_Installation.pdf' title='Install Instructions' target='_blank'>
				" . gettext("Oracle Plugin Installation") . "</a> " . gettext("instructions for monitoring Oracle") . "</div>\n";
            }

            $output = '

	<div class="sectionTitle">' . gettext('Oracle Server') . '</div>
	' . $sanity . '
	<p>' . gettext('Specify the details for connecting to the Oracle server you want to monitor') . '.</p>
			
	<table>

	<tr>
	<td valign="top">
	<label>' . gettext('Address') . ':</label><br class="nobr" />
	</td>
	<td>
<input type="text" size="40" name="address" id="address" value="' . htmlentities($address) . '" class="textfield" /><br class="nobr" />
	' . gettext('The IP address or FQDNS name of the Oracle server') . '.<br><br>
	</td>
	</tr>
	
	<tr>
	<td valign="top">
	<label>' . gettext('Port') . ':</label><br class="nobr" />
	</td>
	<td>
<input type="text" size="5" name="port" id="port" value="' . htmlentities($port) . '" class="textfield" /><br class="nobr" />
	' . gettext('The port to use to connect to the Oracle server') . '.<br><br>
	</td>
	</tr>
    
    <tr>
	<td valign="top">
	<label>' . gettext('SID') . ':</label><br class="nobr" />
	</td>
	<td>
<input type="text" size="5" name="sid" id="sid" value="' . htmlentities($sid) . '" class="textfield" /><br class="nobr" />
	' . gettext('The sid to use to connect to the Oracle server') . '.<br><br>
	</td>
	</tr>

	<tr>
	<td valign="top">
	<label>' . gettext('Username') . ':</label><br class="nobr" />
	</td>
	<td>
<input type="text" size="20" name="username" id="username" value="' . htmlentities($username) . '" class="textfield" /><br class="nobr" />
	' . gettext('The username used to connect to the Oracle server') . '.<br><br>
	</td>
	</tr>

	<tr>
	<td valign="top">
	<label>' . gettext('Password') . ':</label><br class="nobr" />
	</td>
	<td>
<input type="password" size="20" name="password" id="password" value="' . htmlentities($password) . '" class="textfield" /><br class="nobr" />
	' . gettext('The password used to connect to the Oracle server') . '.<br><br>
	</td>
	</tr>

	<tr>
	<td valign="top">
	<label>' . gettext('Database') . ':</label><br class="nobr" />
	</td>
	<td>
<input type="text" size="20" name="database" id="database" value="' . htmlentities($database) . '" class="textfield" /><br class="nobr" />
	' . gettext('The database to connect to on the Oracle server') . '.<br><br>
	</td>
	</tr>


	</table>

			';
            break;

        case CONFIGWIZARD_MODE_VALIDATESTAGE1DATA:

            // get variables that were passed to us
            $address = grab_array_var($inargs, "address", "");
            $port = grab_array_var($inargs, "port", "");
            $sid = grab_array_var($inargs, "sid", "");
            $username = grab_array_var($inargs, "username", "");
            $password = grab_array_var($inargs, "password", "");
            $database = grab_array_var($inargs, "database", "");

            // check for errors
            $errors = 0;
            $errmsg = array();
            if (have_value($address) == false)
                $errmsg[$errors++] = gettext("No address specified.");
            if (have_value($port) == false)
                $errmsg[$errors++] = gettext("No port number specified.");
            if (have_value($username) == false)
                $errmsg[$errors++] = gettext("No username specified.");
            if (have_value($password) == false)
                $errmsg[$errors++] = gettext("No password specified.");
            if (have_value($database) == false)
                $errmsg[$errors++] = gettext("No database specified.");

            if ($errors > 0) {
                $outargs[CONFIGWIZARD_ERROR_MESSAGES] = $errmsg;
                $result = 1;
            }

            break;

        case CONFIGWIZARD_MODE_GETSTAGE2HTML:

            // get variables that were passed to us
            $address = grab_array_var($inargs, "address");
            $port = grab_array_var($inargs, "port", "");
            $username = grab_array_var($inargs, "username", "");
            $password = grab_array_var($inargs, "password", "");
            $sid = grab_array_var($inargs, "sid", "");
            $database = grab_array_var($inargs, "database", "");

            $query = grab_array_var($inargs, "query", 'select count(*) from v\$session where status = "active"');
            $queryname = grab_array_var($inargs, "queryname", "Test Query");
            $result = grab_array_var($inargs, "result", "Expected result");
            $warning = grab_array_var($inargs, "warning", "50");
            $critical = grab_array_var($inargs, "critical", "200");

            $ha = @gethostbyaddr($address);
            if ($ha == "")
                $ha = $address;
            $hostname = grab_array_var($inargs, "hostname", $ha);


            $output = '
			
		<input type="hidden" name="address" value="' . htmlentities($address) . '">
		<input type="hidden" name="port" value="' . htmlentities($port) . '">
        <input type="hidden" name="sid" value="' . htmlentities($sid) . '">
		<input type="hidden" name="username" value="' . htmlentities($username) . '">
		<input type="hidden" name="password" value="' . htmlentities($password) . '">
		<input type="hidden" name="database" value="' . htmlentities($database) . '">

	<div class="sectionTitle">' . gettext('Oracle Server') . '</div>
	
			
	<table>
	
	<tr>
	<td valign="top">
	<label>' . gettext('Address') . ':</label><br class="nobr" />
	</td>
	<td>
<input type="text" size="20" name="address" id="address" value="' . htmlentities($address) . '" class="textfield" disabled/><br class="nobr" />
	</td>
	</tr>

	<tr>
	<td valign="top">
	<label>' . gettext('Host Name') . ':</label><br class="nobr" />
	</td>
	<td>
<input type="text" size="20" name="hostname" id="hostname" value="' . htmlentities($hostname) . '" class="textfield" /><br class="nobr" />
	' . gettext("The name you'd like to have associated with this Oracle server.") . '
	</td>
	</tr>
	
	<tr>
	<td valign="top">
	<label>' . gettext('Port') . ':</label><br class="nobr" />
	</td>
	<td>
<input type="text" size="5" name="port" id="port" value="' . htmlentities($port) . '" class="textfield" disabled/><br class="nobr" />
	</td>
	</tr>

	<tr>
	<td valign="top">
	<label>U' . gettext('sername') . ':</label><br class="nobr" />
	</td>
	<td>
<input type="text" size="20" name="username" id="username" value="' . htmlentities($username) . '" class="textfield" disabled/><br class="nobr" />
	</td>
	</tr>

	<tr>
	<td valign="top">
	<label>' . gettext('Password') . ':</label><br class="nobr" />
	</td>
	<td>
<input type="password" size="20" name="password" id="password" value="' . htmlentities($password) . '" class="textfield" disabled/><br class="nobr" />
	</td>
	</tr>

	<tr>
	<td valign="top">
	<label>' . gettext('Database') . ':</label><br class="nobr" />
	</td>
	<td>
<input type="text" size="20" name="database" id="database" value="' . htmlentities($database) . '" class="textfield" disabled/><br class="nobr" />
	</td>
	</tr>

	</table>


	<div class="sectionTitle">' . gettext('Oracle Query') . '</div>
	
	<p>' . gettext("Specify the details of the query you'd like to monitor") . '.</p>
	
	
	<table>
	
	<tr>
	<td valign="top">
	<label>' . gettext('Query Name') . ':</label><br class="nobr" />
	</td>
	<td>
<input type="text" size="40" name="queryname" id="queryname" value="' . htmlentities($queryname) . '" class="textfield" ><br class="nobr" />
	' . gettext('A friendly name for the SQL query') . '.<br><br>
	</td>
	</tr>
	
	<tr>
	<td valign="top">
	<label>' . gettext('Query') . ':</label><br class="nobr" />
	</td>
	<td>
<input type="text" size="60" name="query" id="query" value="' . htmlentities($query) . '" class="textfield" ><br class="nobr" />
	' . gettext('The SQL query to run. MUST return a number, not a string') . '.<br><br>
	</td>
	</tr>
	
	<tr>
	<td valign="top">
	<label>' . gettext('Warning') . ':</label><br class="nobr" />
	</td>
	<td>
<input type="text" size="4" name="warning" id="warning" value="' . htmlentities($warning) . '" class="textfield" ><br class="nobr" />
	' . gettext('An optional warning threshold to use when checking the result of the SQL query') . '.<br><br>
	</td>
	</tr>
	
	<tr>
	<td valign="top">
	<label>' . gettext('Critical') . ':</label><br class="nobr" />
	</td>
	<td>
<input type="text" size="4" name="critical" id="critical" value="' . htmlentities($critical) . '" class="textfield" ><br class="nobr" />
	' . gettext('An optional critical threshold to use when checking the result of the SQL query') . '.<br><br>
	</td>
	</tr>
	
	</table>
	';
            break;

        case CONFIGWIZARD_MODE_VALIDATESTAGE2DATA:

            // get variables that were passed to us
            $address = grab_array_var($inargs, "address");
            $hostname = grab_array_var($inargs, "hostname");
            $port = grab_array_var($inargs, "port", "");
            $sid = grab_array_var($inargs, "sid", "");
            $username = grab_array_var($inargs, "username", "");
            $password = grab_array_var($inargs, "password", "");
            $database = grab_array_var($inargs, "database", "");

            $query = grab_array_var($inargs, "query", "");
            $queryname = grab_array_var($inargs, "queryname", "");
            $reult = grab_array_var($inargs, "result", "");
            $warning = grab_array_var($inargs, "warning", "");
            $critical = grab_array_var($inargs, "critical", "");


            $services = grab_array_var($inargs, "services", array());
            $serviceargs = grab_array_var($inargs, "serviceargs", array());


            // check for errors
            $errors = 0;
            $errmsg = array();
            if (is_valid_host_name($hostname) == false)
                $errmsg[$errors++] = gettext("Invalid host name.");
            if (is_valid_service_name($queryname) == false)
                $errmsg[$errors++] = gettext("Invalid query name") . " '" . $queryname . "'.";

            if ($errors > 0) {
                $outargs[CONFIGWIZARD_ERROR_MESSAGES] = $errmsg;
                $result = 1;
            }

            break;


        case CONFIGWIZARD_MODE_GETSTAGE3HTML:

            // get variables that were passed to us
            $address = grab_array_var($inargs, "address");
            $hostname = grab_array_var($inargs, "hostname");
            $port = grab_array_var($inargs, "port", "");
            $sid = grab_array_var($inargs, "sid", "");
            $username = grab_array_var($inargs, "username", "");
            $password = grab_array_var($inargs, "password", "");
            $database = grab_array_var($inargs, "database", "");

            $query = grab_array_var($inargs, "query", "");
            $queryname = grab_array_var($inargs, "queryname", "");
            $result = grab_array_var($inargs, "result", "");
            $warning = grab_array_var($inargs, "warning", "");
            $critical = grab_array_var($inargs, "critical", "");

            $services_serial = grab_array_var($inargs, "services_serial");
            if ($services_serial != "")
                $services = unserialize(base64_decode($services_serial));
            else
                $services = grab_array_var($inargs, "services");
            $serviceargs_serial = grab_array_var($inargs, "serviceargs_serial");
            if ($serviceargs_serial != "")
                $serviceargs = unserialize(base64_decode($serviceargs_serial));
            else
                $serviceargs = grab_array_var($inargs, "serviceargs");

            $output = '
			
		<input type="hidden" name="address" value="' . htmlentities($address) . '">
		<input type="hidden" name="hostname" value="' . htmlentities($hostname) . '">
		<input type="hidden" name="port" value="' . htmlentities($port) . '">
        <input type="hidden" name="sid" value="' . htmlentities($sid) . '">
		<input type="hidden" name="username" value="' . htmlentities($username) . '">
		<input type="hidden" name="password" value="' . htmlentities($password) . '">
		<input type="hidden" name="database" value="' . htmlentities($database) . '">
		<input type="hidden" name="query" value="' . htmlentities($query) . '">
		<input type="hidden" name="queryname" value="' . htmlentities($queryname) . '">
		<input type="hidden" name="result" value="' . htmlentities($result) . '">
		<input type="hidden" name="warning" value="' . htmlentities($warning) . '">
		<input type="hidden" name="critical" value="' . htmlentities($critical) . '">
		<input type="hidden" name="services_serial" value="' . base64_encode(serialize($services)) . '">
		<input type="hidden" name="serviceargs_serial" value="' . base64_encode(serialize($serviceargs)) . '">
		
		<!--SERVICES=' . serialize($services) . '<BR>
		SERVICEARGS=' . serialize($serviceargs) . '<BR>-->
		
			';
            break;

        case CONFIGWIZARD_MODE_VALIDATESTAGE3DATA:


            break;

        case CONFIGWIZARD_MODE_GETFINALSTAGEHTML:


            $output = '
			';
            break;

        case CONFIGWIZARD_MODE_GETOBJECTS:

            $hostname = grab_array_var($inargs, "hostname", "");
            $address = grab_array_var($inargs, "address", "");
            $hostaddress = $address;
            $port = grab_array_var($inargs, "port", "");
            $sid = grab_array_var($inargs, "sid", "");
            $username = grab_array_var($inargs, "username", "");
            $password = grab_array_var($inargs, "password", "");
            $database = grab_array_var($inargs, "database", "");

            $query = grab_array_var($inargs, "query", "");
            $queryname = grab_array_var($inargs, "queryname", "");
            $result = grab_array_var($inargs, "result", "");
            $warning = grab_array_var($inargs, "warning", "");
            $critical = grab_array_var($inargs, "critical", "");

            $services_serial = grab_array_var($inargs, "services_serial", "");
            $serviceargs_serial = grab_array_var($inargs, "serviceargs_serial", "");

            $services = unserialize(base64_decode($services_serial));
            $serviceargs = unserialize(base64_decode($serviceargs_serial));

            /*
            echo "SERVICES<BR>";
            print_r($services);
            echo "<BR>";
            echo "SERVICEARGS<BR>";
            print_r($serviceargs);
            echo "<BR>";
            */

            // save data for later use in re-entrance
            $meta_arr = array();
            $meta_arr["hostname"] = $hostname;
            $meta_arr["address"] = $address;
            $meta_arr["port"] = $port;
            $meta_arr["sid"] = $sid;
            $meta_arr["username"] = $username;
            $meta_arr["password"] = $password;
            $meta_arr["database"] = $database;
            $meta_arr["query"] = $query;
            $meta_arr["queryname"] = $queryname;
            $meta_arr["result"] = $result;
            $meta_arr["warning"] = $warning;
            $meta_arr["critical"] = $critical;
            $meta_arr["services"] = $services;
            $meta_arr["serivceargs"] = $serviceargs;
            save_configwizard_object_meta($wizard_name, $hostname, "", $meta_arr);

            $objs = array();

            if (!host_exists($hostname)) {
                $objs[] = array(
                    "type" => OBJECTTYPE_HOST,
                    "use" => "xiwizard_oraclequery_host",
                    "host_name" => $hostname,
                    "address" => $hostaddress,
                    "icon_image" => "oraclequery.png",
                    "statusmap_image" => "oraclequery.png",
                    "_xiwizard" => $wizard_name,
                );
            }

            // common plugin opts
            if (have_value($sid) == false) {
                $commonopts = "--connect '{$address}:{$port}' --username '{$username}' --password '{$password}' ";
                $add_to_description = "";
            } else {
                $commonopts = "--connect '{$address}:{$port}/{$sid}' --username '{$username}' --password '{$password}' ";
                $add_to_description = $sid . " ";
            }

            $pluginopts = $commonopts . '--mode sql --name="' . html_entity_decode($query) . '"';
            if ($warning != "")
                $pluginopts .= " --warning " . $warning;
            if ($critical != "")
                $pluginopts .= " --critical " . $critical;


            $objs[] = array(
                "type" => OBJECTTYPE_SERVICE,
                "host_name" => $hostname,
                "service_description" => "Oracle Query - " . $queryname,
                "use" => "xiwizard_oraclequery_service",
                "check_command" => "check_xi_oraclequery!" . $pluginopts,
                "_xiwizard" => $wizard_name,
            );

            //echo "OBJECTS:<BR>";
            //print_r($objs);
            //exit();

            // return the object definitions to the wizard
            $outargs[CONFIGWIZARD_NAGIOS_OBJECTS] = $objs;

            break;

        default:
            break;
    }

    return $output;
}


?>